@@ -113,11 +113,12 @@ from chalk.features.tag import DeploymentId
 from chalk.importer import CHALK_IMPORT_FLAG
 from chalk.ml import LocalSourceConfig, ModelEncoding, ModelRunCriterion, ModelType, SourceConfig
 from chalk.ml.model_file_transfer import ModelFileUploader
+from chalk.ml.utils import ModelClass
 from chalk.parsed._proto.utils import datetime_to_proto_timestamp, value_to_proto
 from chalk.utils import df_utils
 from chalk.utils.df_utils import record_batch_to_arrow_ipc
 from chalk.utils.grpc import AuthenticatedChalkClientInterceptor, TokenRefresher, UnauthenticatedChalkClientInterceptor
-from chalk.utils.tracing import add_trace_headers
+from chalk.utils.tracing import add_trace_headers, safe_trace
 
 if TYPE_CHECKING:
     from pyarrow import RecordBatch, Table
@@ -795,6 +796,7 @@ class ChalkGRPCClient:
             request_timeout=request_timeout,
             headers=headers,
             query_context=_validate_context_dict(query_context),
+            trace=trace,
         )
         return OnlineQueryConverter.online_query_bulk_response_decode_to_single(bulk_response)
 
@@ -822,37 +824,38 @@ class ChalkGRPCClient:
         query_context: Mapping[str, Union[str, int, float, bool, None]] | None = None,
         trace: bool = False,
     ) -> online_query_pb2.OnlineQueryBulkResponse:
-        request = self._make_query_bulk_request(
-            input={k: [v] for k, v in input.items()},
-            output=output,
-            now=[now] if now is not None else [],
-            staleness=staleness or {},
-            tags=tags or (),
-            correlation_id=correlation_id,
-            query_name=query_name,
-            query_name_version=query_name_version,
-            include_meta=include_meta,
-            meta=meta or {},
-            explain=explain,
-            store_plan_stages=store_plan_stages,
-            value_metrics_tag_by_features=value_metrics_tag_by_features,
-            encoding_options=encoding_options,
-            required_resolver_tags=required_resolver_tags or (),
-            planner_options=planner_options or {},
-            query_context=query_context,
-        )
-        if trace:
-            extra_headers: dict[str, str] = {}
-            extra_headers = add_trace_headers(extra_headers)
-            headers = _merge_headers(extra_headers, headers)
-        metadata = _canonicalize_headers(headers)
-        return self._stub_refresher.call_query_stub(
-            lambda x: x.OnlineQueryBulk(
-                request,
-                timeout=request_timeout,
-                metadata=metadata,
+        with safe_trace("_online_query_grpc_request"):
+            request = self._make_query_bulk_request(
+                input={k: [v] for k, v in input.items()},
+                output=output,
+                now=[now] if now is not None else [],
+                staleness=staleness or {},
+                tags=tags or (),
+                correlation_id=correlation_id,
+                query_name=query_name,
+                query_name_version=query_name_version,
+                include_meta=include_meta,
+                meta=meta or {},
+                explain=explain,
+                store_plan_stages=store_plan_stages,
+                value_metrics_tag_by_features=value_metrics_tag_by_features,
+                encoding_options=encoding_options,
+                required_resolver_tags=required_resolver_tags or (),
+                planner_options=planner_options or {},
+                query_context=query_context,
+            )
+            if trace:
+                extra_headers: dict[str, str] = {}
+                extra_headers = add_trace_headers(extra_headers)
+                headers = _merge_headers(extra_headers, headers)
+            metadata = _canonicalize_headers(headers)
+            return self._stub_refresher.call_query_stub(
+                lambda x: x.OnlineQueryBulk(
+                    request,
+                    timeout=request_timeout,
+                    metadata=metadata,
+                )
             )
-        )
 
     def online_query_bulk(
         self,
@@ -1471,6 +1474,7 @@ class ChalkGRPCClient:
         self,
         name: str,
         model_type: Optional[ModelType] = None,
+        model_class: Optional[ModelClass] = None,
         model_encoding: Optional[ModelEncoding] = None,
         aliases: Optional[List[str]] = None,
         model: Optional[Any] = None,
@@ -1568,6 +1572,9 @@ class ChalkGRPCClient:
             if model_type is None:
                 model_type = model_serializer.model_type
 
+            if model_class is None:
+                model_class = model_serializer.model_class
+
             if model is not None:
                 inferred_input_schema, inferred_output_schema = model_serializer.infer_input_output_schemas(
                     model, model_type
@@ -1608,9 +1615,29 @@ class ChalkGRPCClient:
                             "Failed to register model. Please specify a model encoding if using model_paths."
                         )
 
+                # Auto-convert ONNX list schemas to dict format if needed
+                if model_type == ModelType.ONNX:
+                    input_schema = model_serializer.convert_onnx_list_schema_to_dict(input_schema, model, is_input=True)
+                    output_schema = model_serializer.convert_onnx_list_schema_to_dict(
+                        output_schema, model, is_input=False
+                    )
+
                 input_model_schema = model_serializer.convert_schema(input_schema)
                 output_model_schema = model_serializer.convert_schema(output_schema)
 
+                # Final validation: ONNX models must use tabular schemas
+                if model_type == ModelType.ONNX:
+                    if input_model_schema is not None and not input_model_schema.HasField("tabular"):
+                        raise ValueError(
+                            "ONNX models must be registered with tabular input schema (dict format). "
+                            + "Use dict format like {'input': Tensor[...]} instead of list format."
+                        )
+                    if output_model_schema is not None and not output_model_schema.HasField("tabular"):
+                        raise ValueError(
+                            "ONNX models must be registered with tabular output schema (dict format). "
+                            + "Use dict format like {'output': Vector[...]} instead of list format."
+                        )
+
                 all_files_to_process, model_file_names = model_file_uploader.prepare_file_mapping(
                     model_paths, additional_files
                 )
@@ -1640,6 +1667,7 @@ class ChalkGRPCClient:
                                         for file in additional_files_upload_paths
                                     ],
                                     model_type=model_type,
+                                    model_class=model_class,
                                     model_encoding=model_encoding,
                                     model_signature=_model_artifact_pb2.ModelSignature(
                                         inputs=input_model_schema,
