@@ -48,6 +48,7 @@ from chalk.integrations.catalogs.base_catalog import BaseCatalog
 from chalk.utils.df_utils import read_parquet
 from chalk.utils.log_with_context import get_logger
 from chalk.utils.missing_dependency import missing_dependency_exception
+from chalk.utils.pl_helpers import apply_compat, polars_group_by_instead_of_groupby
 from chalk.utils.threading import DEFAULT_IO_EXECUTOR
 
 if TYPE_CHECKING:
@@ -541,14 +542,19 @@ def _extract_df_columns(
             for fqn in unique_features
         ]
 
-        df = df.groupby("pkey").agg(cols)
+        if polars_group_by_instead_of_groupby:
+            df = df.group_by("pkey").agg(cols)
+        else:
+            df = df.groupby("pkey").agg(cols)  # pyright: ignore
         decoded_stmts: List[pl.Expr] = []
         for col in df.columns:
             if col == "pkey":
                 continue
             else:
                 decoded_stmts.append(
-                    pl.col(col).apply(_json_decode, return_dtype=Feature.from_root_fqn(col).converter.polars_dtype)
+                    apply_compat(
+                        pl.col(col), _json_decode, return_dtype=Feature.from_root_fqn(col).converter.polars_dtype
+                    )
                 )
         df = df.select(decoded_stmts)
         # it might be a good idea to remember that we used to rename this __id__ column to the primary key
@@ -571,7 +577,7 @@ def _extract_df_columns(
                 # Assuming that the only datetime column is for timestamps
                 decoded_stmts.append(to_utc(df, col, pl.col(col)))
             else:
-                decoded_stmts.append(pl.col(col).apply(_json_decode, return_dtype=dtype))
+                decoded_stmts.append(apply_compat(pl.col(col), _json_decode, return_dtype=dtype))
         elif version in (DatasetVersion.NATIVE_DTYPES, DatasetVersion.NATIVE_COLUMN_NAMES):
             # We already decoded the column names so matching against the fqn
             if col == CHALK_TS_FEATURE or col == OBSERVED_AT_FEATURE:
@@ -1338,12 +1344,14 @@ class DatasetRevisionImpl(DatasetRevision):
                         actual_args.append(
                             DataFrame(
                                 pl.DataFrame([pl.Series(col_name, [], dtype=raw_input_df.schema[col_name])])
-                                .explode(pl.all())
+                                .explode(col_name)
                                 .unnest(col_name)
                             )
                         )
                     else:
-                        actual_args.append(DataFrame(has_many_input_df.explode(pl.all()).unnest(col_name)))
+                        actual_args.append(
+                            DataFrame(has_many_input_df.explode(has_many_input_df.columns).unnest(col_name))
+                        )
                 else:
                     value = args[i]
                     if isinstance(input, Feature):
