@@ -7,10 +7,9 @@ import traceback
 import uuid
 from datetime import datetime, timedelta
 from enum import Enum, IntEnum
-from typing import TYPE_CHECKING, Any, Dict, List, Literal, Mapping, Optional, Sequence, Tuple, Union
+from typing import TYPE_CHECKING, Any, Dict, List, Literal, Mapping, Optional, Sequence, Tuple, TypeAlias, Union
 
 import numpy as np
-from typing_extensions import TypeAlias
 
 from chalk.byte_transmit.model import ByteBaseModel, ByteDict
 from chalk.client._internal_models.models import OfflineQueryGivensVersion
