@@ -77,7 +77,15 @@ MODEL_SERIALIZERS = {
     ModelType.ONNX: ModelSerializationConfig(
         filename="model.onnx",
         encoding=ModelEncoding.PROTOBUF,
-        serialize_fn=lambda model, path: model.save_model(path),
+        serialize_fn=lambda model, path: ModelSerializer.with_import(
+            "onnx",
+            lambda onnx: onnx.save(
+                # Unwrap model if it has a _model attribute (e.g., wrapped ONNX models)
+                model._model if hasattr(model, "_model") else model,
+                path,
+            ),
+            "Please install onnx to save ONNX models.",
+        ),
     ),
 }
 
@@ -86,7 +94,13 @@ class ModelSerializer:
     def __init__(self, model: Any, model_type: Optional[ModelType]):
         self._temp_files: List[str] = []
         self.model = model
-        self.model_type = model_type if model_type is not None else ModelAttributeExtractor.infer_model_type(model)
+        if model_type is not None:
+            self.model_type = model_type
+            self.model_class = None
+        else:
+            model_type, model_class = ModelAttributeExtractor.infer_model_type(model)
+            self.model_type = model_type
+            self.model_class = model_class
 
         if self.model_type is None:
             raise ValueError("Unable to infer model type from object and no type given.")
@@ -275,7 +289,15 @@ class ModelSerializer:
         tensor_schema = _model_artifact_pb2.TensorSchema()
 
         for shape, dtype in tensor_specs:
-            if not isinstance(dtype, pa.DataType):
+            # Handle Chalk Tensor types
+            if hasattr(dtype, "__mro__") and any("Tensor" in base.__name__ for base in dtype.__mro__):
+                # Extract shape and dtype from Tensor type
+                if hasattr(dtype, "shape") and hasattr(dtype, "dtype"):
+                    shape = dtype.shape
+                    pa_dtype = dtype.dtype
+                else:
+                    raise ValueError(f"Tensor type is missing shape or dtype attributes")
+            elif not isinstance(dtype, pa.DataType):
                 if dtype == str:
                     pa_dtype = pa.string()
                 elif dtype == int:
@@ -299,12 +321,73 @@ class ModelSerializer:
 
         return tensor_schema
 
+    @staticmethod
+    def convert_onnx_list_schema_to_dict(schema: Any, model: Any, is_input: bool = True) -> Any:
+        """Convert list-based schema to dict-based schema for ONNX models.
+
+        Args:
+            schema: The schema (list or dict)
+            model: The ONNX model (ModelProto or wrapped)
+            is_input: True for input schema, False for output schema
+
+        Returns:
+            Dict-based schema with field names from ONNX model
+        """
+        if not isinstance(schema, list):
+            return schema
+
+        try:
+            import onnx  # type: ignore[reportMissingImports]
+        except ImportError:
+            raise ValueError("onnx package is required to convert list schemas for ONNX models")
+
+        # Unwrap model if needed
+        onnx_model = model._model if hasattr(model, "_model") else model
+
+        if not isinstance(onnx_model, onnx.ModelProto):
+            raise ValueError(
+                f"ONNX models must be registered with tabular schema (dict format). "
+                + f"Use dict format like {{'input': Tensor[...]}} instead of list format."
+            )
+
+        # Get input/output names from ONNX model
+        if is_input:
+            names = [inp.name for inp in onnx_model.graph.input]
+            schema_type = "input"
+        else:
+            names = [out.name for out in onnx_model.graph.output]
+            schema_type = "output"
+
+        if len(names) != len(schema):
+            raise ValueError(f"ONNX model has {len(names)} {schema_type}s but schema has {len(schema)} entries")
+
+        # Convert to dict format
+        return {name: spec for name, spec in zip(names, schema)}
+
     @staticmethod
     def convert_schema(schema: Any) -> Optional[_model_artifact_pb2.ModelSchema]:
         model_schema = _model_artifact_pb2.ModelSchema()
         if schema is not None:
             if isinstance(schema, dict):
-                model_schema.tabular.CopyFrom(ModelSerializer.build_tabular_schema(schema))
+                # Convert Tensor/Vector types to their PyArrow types for tabular schema
+                converted_schema = {}
+                for col_name, dtype in schema.items():
+                    if hasattr(dtype, "__mro__") and any("Tensor" in base.__name__ for base in dtype.__mro__):
+                        # Use Tensor's to_pyarrow_dtype() method to convert to Arrow type
+                        if hasattr(dtype, "to_pyarrow_dtype"):
+                            converted_schema[col_name] = dtype.to_pyarrow_dtype()
+                        else:
+                            raise ValueError(f"Tensor type for '{col_name}' is missing to_pyarrow_dtype method")
+                    elif hasattr(dtype, "__mro__") and any("Vector" in base.__name__ for base in dtype.__mro__):
+                        # Vector already has a .dtype attribute that's a PyArrow type
+                        if hasattr(dtype, "dtype"):
+                            converted_schema[col_name] = dtype.dtype
+                        else:
+                            raise ValueError(f"Vector type for '{col_name}' is missing dtype attribute")
+                    else:
+                        converted_schema[col_name] = dtype
+
+                model_schema.tabular.CopyFrom(ModelSerializer.build_tabular_schema(converted_schema))
             elif isinstance(schema, list):
                 model_schema.tensor.CopyFrom(ModelSerializer.build_tensor_schema(schema))
             else:
