@@ -19,11 +19,10 @@ from typing import (
     Set,
     Tuple,
     Type,
+    TypeGuard,
     cast,
 )
 
-from typing_extensions import TypeGuard
-
 from chalk._lsp._class_finder import get_class_ast
 from chalk.features.feature_wrapper import FeatureWrapper, unwrap_feature
 from chalk.utils import notebook
