@@ -3,6 +3,7 @@ from __future__ import annotations
 import collections.abc
 import contextvars
 import dataclasses
+import inspect
 from datetime import datetime, timedelta, timezone
 from types import TracebackType
 from typing import TYPE_CHECKING, Any, Hashable, Set, Type
@@ -11,6 +12,10 @@ if TYPE_CHECKING:
     from chalk.features.feature_field import Feature
 
 
+class ClauseJoinWithAndException(Exception):
+    ...
+
+
 class Filter:
     def __init__(self, lhs: Any, operation: str, rhs: Any):
         from chalk.features.feature_wrapper import FeatureWrapper, unwrap_feature
@@ -62,6 +67,12 @@ class Filter:
         return f"Filter({self.lhs} {self.operation} {self.rhs})"
 
     def __bool__(self):
+        frame = inspect.currentframe()
+        if frame and frame.f_back:
+            frame = frame.f_back
+            if frame.f_code.co_name == "<lambda>":
+                raise ClauseJoinWithAndException()  # must join with '&', not 'and'
+        del frame
         return bool(self.rhs == self.lhs)
 
 
@@ -111,7 +122,9 @@ def before(
 ) -> Any:
     """The function `before` can be used with `DataFrame` to compute windowed features.
 
-    Before filters a `DataFrame` relative to the current time in context.
+    `before` filters a `DataFrame` relative to the current time in context such that
+    if the `before` filter is defined as `now - {time_window}`, the filter will include
+    all features with timestamps t where `t <= now - {time_window}`.
     This time could be in the past if you’re using an offline resolver.
     Using window functions ensures that you maintain point-in-time correctness.
 
@@ -184,7 +197,9 @@ def after(
 ) -> Any:
     """The function `after` can be used with `DataFrame` to compute windowed features.
 
-    After filters a `DataFrame` relative to the current time in context.
+    `after` filters a `DataFrame` relative to the current time in context, such that if
+    the `after` filter is defined as `now - {time_window}`, the filter will include
+    all features with timestamps t where `now - {time_window} <= t <= now`.
     This time could be in the past if you’re using an offline resolver.
     Using window functions ensures that you maintain point-in-time correctness.
 
@@ -252,6 +267,12 @@ def get_filter_now() -> datetime:
     return _FILTER_NOW_CONTEXT_STACK.get(datetime.now(tz=timezone.utc))
 
 
+def time_is_frozen() -> bool:
+    rt = datetime.now(tz=timezone.utc)
+    now = _FILTER_NOW_CONTEXT_STACK.get(rt)
+    return now != rt
+
+
 class freeze_time:
     def __init__(self, at: datetime):
         """
@@ -264,6 +285,8 @@ class freeze_time:
 
         Examples
         --------
+        >>> from chalk.features import online, DataFrame
+        >>> from datetime import datetime, timedelta, timezone
         >>> @online
         ... def get_average_spend_30d(
         ...     spend: User.cards[after(days_ago=30)],
