@@ -4,12 +4,15 @@ import asyncio
 import inspect
 import logging
 import time  # Added for measuring duration
-from typing import Any, Callable, Iterable, Optional, Set, Tuple, TypeAlias, Union, overload
+from typing import TYPE_CHECKING, Any, Callable, Iterable, Optional, Set, Tuple, TypeAlias, Union, overload
 
 from chalk.features.tag import Environments
 from chalk.utils.collections import ensure_tuple
 from chalk.utils.log_with_context import get_logger
 
+if TYPE_CHECKING:
+    from chalk.features.resolver import ResourceHint
+
 HookFn: TypeAlias = Callable[[], Any]
 
 
@@ -43,21 +46,31 @@ class Hook:
     venv: Optional[str]
     fn: HookFn
     filename: str
+    resource_hint: Optional["ResourceHint"]
+    resource_group: Optional[str]
 
     def __init__(
-        self, fn: HookFn, filename: str, environment: Optional[Environments] = None, venv: Optional[str] = None
+        self,
+        fn: HookFn,
+        filename: str,
+        environment: Optional[Environments] = None,
+        venv: Optional[str] = None,
+        resource_hint: Optional["ResourceHint"] = None,
+        resource_group: Optional[str] = None,
     ):
         super().__init__()
         self.fn = fn
         self.filename = filename
         self.environment = None if environment is None else ensure_tuple(environment)
         self.venv = venv
+        self.resource_hint = resource_hint
+        self.resource_group = resource_group
 
     def __call__(self):
         return self.fn()
 
     def __repr__(self):
-        return f'Hook(filename={self.filename}, fn={self.fn.__name__}", environment={str(self.environment)}, venv={self.venv})'
+        return f'Hook(filename={self.filename}, fn={self.fn.__name__}", environment={str(self.environment)}, venv={self.venv}, resource_hint={self.resource_hint}, resource_group={self.resource_group})'
 
     @classmethod
     async def async_run_all_before_all(cls, environment: str, venv: Optional[str] = None) -> None:
@@ -75,17 +88,46 @@ def before_all(fn: HookFn, /) -> Hook:
 
 @overload
 def before_all(
-    fn: None = None, /, environment: Optional[Environments] = None, venv: Optional[str] = None
+    fn: HookFn,
+    /,
+    environment: Optional[Environments] = None,
+    venv: Optional[str] = None,
+    resource_hint: Optional["ResourceHint"] = None,
+    resource_group: Optional[str] = None,
+) -> Hook:
+    ...
+
+
+@overload
+def before_all(
+    fn: None = None,
+    /,
+    environment: Optional[Environments] = None,
+    venv: Optional[str] = None,
+    resource_hint: Optional["ResourceHint"] = None,
+    resource_group: Optional[str] = None,
 ) -> Callable[[HookFn], Hook]:
     ...
 
 
 def before_all(
-    fn: Optional[HookFn] = None, /, environment: Optional[Environments] = None, venv: Optional[str] = None
+    fn: Optional[HookFn] = None,
+    /,
+    environment: Optional[Environments] = None,
+    venv: Optional[str] = None,
+    resource_hint: Optional["ResourceHint"] = None,
+    resource_group: Optional[str] = None,
 ) -> Union[Hook, Callable[[HookFn], Hook]]:
     def decorator(f: HookFn):
         caller_filename = inspect.getsourcefile(f) or "unknown_file"
-        hook = Hook(fn=f, filename=caller_filename, environment=environment, venv=venv)
+        hook = Hook(
+            fn=f,
+            filename=caller_filename,
+            environment=environment,
+            venv=venv,
+            resource_hint=resource_hint,
+            resource_group=resource_group,
+        )
         Hook.before_all.add(hook)
         return hook
 
@@ -93,23 +135,52 @@ def before_all(
 
 
 @overload
-def after_all(fn: HookFn, /, environment: Optional[Environments] = None, venv: Optional[str] = None) -> Hook:
+def after_all(fn: HookFn, /) -> Hook:
     ...
 
 
 @overload
 def after_all(
-    fn: None = None, /, environment: Optional[Environments] = None, venv: Optional[str] = None
+    fn: HookFn,
+    /,
+    environment: Optional[Environments] = None,
+    venv: Optional[str] = None,
+    resource_hint: Optional["ResourceHint"] = None,
+    resource_group: Optional[str] = None,
+) -> Hook:
+    ...
+
+
+@overload
+def after_all(
+    fn: None = None,
+    /,
+    environment: Optional[Environments] = None,
+    venv: Optional[str] = None,
+    resource_hint: Optional["ResourceHint"] = None,
+    resource_group: Optional[str] = None,
 ) -> Callable[[HookFn], Hook]:
     ...
 
 
 def after_all(
-    fn: Optional[HookFn] = None, /, environment: Optional[Environments] = None, venv: Optional[str] = None
+    fn: Optional[HookFn] = None,
+    /,
+    environment: Optional[Environments] = None,
+    venv: Optional[str] = None,
+    resource_hint: Optional["ResourceHint"] = None,
+    resource_group: Optional[str] = None,
 ) -> Union[Hook, Callable[[HookFn], Hook]]:
     def decorator(f: HookFn):
         caller_filename = inspect.getsourcefile(f) or "unknown_file"
-        hook = Hook(fn=f, filename=caller_filename, environment=environment, venv=venv)
+        hook = Hook(
+            fn=f,
+            filename=caller_filename,
+            environment=environment,
+            venv=venv,
+            resource_hint=resource_hint,
+            resource_group=resource_group,
+        )
         Hook.after_all.add(hook)
         return hook
 
