@@ -17,6 +17,7 @@ from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Optional, Tuple, Ty
 
 import pyarrow as pa
 
+import chalk.functions as F
 from chalk._lsp.error_builder import DiagnosticBuilder, LSPErrorBuilder
 from chalk.features import Feature, Features, FeatureSetBase, Filter, Vector, unwrap_feature
 from chalk.features.feature_field import WindowConfigResolved
@@ -1442,15 +1443,26 @@ def _supplemental_validate_underscore_expression(
 
     # TODO: Dominic - impl for UnderscoreCall args (we need some special casing for aggregate functions that take in UnderscoreItems)
     if isinstance(underscore, UnderscoreCall):
+        if not isinstance(underscore._chalk__parent, UnderscoreAttr):
+            # we only support calls on attrs, ie _.a.some_attr(*args, **kwargs)
+            raise _UnderscoreValidationError(f"Cannot call non-attribute {underscore._chalk__parent}.")
         caller = underscore._chalk__parent._chalk__parent
+        op_name = underscore._chalk__parent._chalk__attr
+
+        if (op := getattr(F, op_name, None)) is not None:
+            if getattr(op, "_chalk__method_chaining_predicate", lambda _: True)(underscore):
+                return _supplemental_validate_underscore_expression(
+                    state,
+                    class_namespace=class_namespace,
+                    underscore=op(caller, *underscore._chalk__args, **underscore._chalk__kwargs),
+                )
+
         maybe_parent_result = _supplemental_validate_underscore_expression(
             state=state,
             class_namespace=class_namespace,
             underscore=caller,
         )
-        if not isinstance(underscore._chalk__parent, UnderscoreAttr):
-            return None  # TODO: Dominic - is this ever valid?
-        if underscore._chalk__parent._chalk__attr == "where":
+        if op_name == "where":
             if maybe_parent_result is None:
                 return None
             if not isinstance(maybe_parent_result, _HasManyNamespaceExpr) or not isinstance(caller, UnderscoreItem):
@@ -1475,9 +1487,17 @@ def _supplemental_validate_underscore_expression(
                     raise _UnderscoreValidationError(
                         f"the input '{arg!r}' is a feature namespace '{expr.namespace}' which cannot be used as a scalar value"
                     )
+            return None
 
         return None
 
+        # TODO: check that op_name is a supported agg or .agg/.group_by/etc
+        # if op_name in supported_aggs:
+        #     # TODO: typechecking for agg fns
+        #     return None
+        #
+        # raise _UnderscoreValidationError(f"unrecognized function '{op_name}' in expression '{underscore}'")
+
     if isinstance(underscore, UnderscoreItem):
         parent_result = _supplemental_validate_underscore_expression(
             state=state,
