@@ -2,10 +2,11 @@ from __future__ import annotations
 
 from chalk.ml.model_file_transfer import FileInfo, HFSourceConfig, LocalSourceConfig, S3SourceConfig, SourceConfig
 from chalk.ml.model_reference import ModelReference
-from chalk.ml.utils import ModelEncoding, ModelRunCriterion, ModelType
+from chalk.ml.utils import ModelClass, ModelEncoding, ModelRunCriterion, ModelType
 
 __all__ = (
     "ModelType",
+    "ModelClass",
     "ModelEncoding",
     "ModelReference",
     "SourceConfig",
