@@ -3,12 +3,22 @@ from __future__ import annotations
 import inspect
 import os
 from datetime import datetime
+from typing import TYPE_CHECKING
 
 from chalk.ml.model_version import ModelVersion
-from chalk.ml.utils import get_model_spec, get_registry_metadata_file, model_encoding_from_proto, model_type_from_proto
+from chalk.ml.utils import (
+    ModelClass,
+    get_model_spec,
+    get_registry_metadata_file,
+    model_encoding_from_proto,
+    model_type_from_proto,
+)
 from chalk.utils.object_inspect import get_source_object_starting
 from chalk.utils.source_parsing import should_skip_source_code_parsing
 
+if TYPE_CHECKING:
+    from chalk.features.resolver import ResourceHint
+
 
 class ModelReference:
     def __init__(
@@ -18,6 +28,8 @@ class ModelReference:
         version: int | None = None,
         alias: str | None = None,
         as_of_date: datetime | None = None,
+        resource_hint: "ResourceHint | None" = None,
+        resource_group: str | None = None,
     ):
         """Specifies the model version that should be loaded into the deployment.
 
@@ -68,6 +80,8 @@ class ModelReference:
         self.as_of_date = as_of_date
         self.alias = alias
         self.identifier = identifier
+        self.resource_hint = resource_hint
+        self.resource_group = resource_group
 
         self.filename = filename
         self.source_line_start = source_line_start
@@ -101,6 +115,11 @@ class ModelReference:
                 identifier=identifier,
                 model_type=model_type_from_proto(model_artifact_metadata.spec.model_type),
                 model_encoding=model_encoding_from_proto(model_artifact_metadata.spec.model_encoding),
+                model_class=ModelClass(model_artifact_metadata.spec.model_class)
+                if model_artifact_metadata.spec.model_class
+                else None,
+                resource_hint=resource_hint,
+                resource_group=resource_group,
             )
 
             from chalk.features.hooks import before_all
@@ -108,14 +127,22 @@ class ModelReference:
             def hook():
                 mv.load_model()
 
-            before_all(hook)
+            before_all(hook, resource_hint=resource_hint, resource_group=resource_group)
 
             self.model_version = mv
         else:
-            self.model_version = ModelVersion(name=name, identifier=identifier)
+            self.model_version = ModelVersion(
+                name=name, identifier=identifier, resource_hint=resource_hint, resource_group=resource_group
+            )
 
     @classmethod
-    def as_of(cls, name: str, when: datetime) -> ModelVersion:
+    def as_of(
+        cls,
+        name: str,
+        when: datetime,
+        resource_hint: "ResourceHint | None" = None,
+        resource_group: str | None = None,
+    ) -> ModelVersion:
         """Creates a ModelReference for a specific point in time.
 
         Parameters
@@ -124,6 +151,11 @@ class ModelReference:
             The name of the model.
         when
             The datetime to use for creating the model version identifier.
+        resource_hint
+            Whether this model loading is bound by CPU, I/O, or GPU.
+        resource_group
+            The resource group for the model: this is used to isolate execution
+            onto a separate pod (or set of nodes), such as on a GPU-enabled node.
 
         Returns
         -------
@@ -135,13 +167,20 @@ class ModelReference:
         >>> import datetime
         >>> timestamp = datetime.datetime(2023, 10, 15, 14, 30, 0)
         >>> model = ModelReference.as_of("fraud_model", timestamp)
+        >>> model = ModelReference.as_of("fraud_model", timestamp, resource_hint="gpu", resource_group="gpu-group")
         """
 
-        mr = ModelReference(name=name, as_of_date=when)
+        mr = ModelReference(name=name, as_of_date=when, resource_hint=resource_hint, resource_group=resource_group)
         return mr.model_version
 
     @classmethod
-    def from_version(cls, name: str, version: int) -> ModelVersion:
+    def from_version(
+        cls,
+        name: str,
+        version: int,
+        resource_hint: "ResourceHint | None" = None,
+        resource_group: str | None = None,
+    ) -> ModelVersion:
         """Creates a ModelReference using a numeric version identifier.
 
         Parameters
@@ -150,6 +189,11 @@ class ModelReference:
             The name of the model.
         version
             The version number. Must be a non-negative integer.
+        resource_hint
+            Whether this model loading is bound by CPU, I/O, or GPU.
+        resource_group
+            The resource group for the model: this is used to isolate execution
+            onto a separate pod (or set of nodes), such as on a GPU-enabled node.
 
         Returns
         -------
@@ -164,15 +208,22 @@ class ModelReference:
         Examples
         --------
         >>> model = ModelReference.from_version("fraud_model", 1)
+        >>> model = ModelReference.from_version("fraud_model", 1, resource_hint="gpu", resource_group="gpu-group")
         """
         if version < 0:
             raise ValueError("Version number must be a non-negative integer.")
 
-        mr = ModelReference(name=name, version=version)
+        mr = ModelReference(name=name, version=version, resource_hint=resource_hint, resource_group=resource_group)
         return mr.model_version
 
     @classmethod
-    def from_alias(cls, name: str, alias: str) -> ModelVersion:
+    def from_alias(
+        cls,
+        name: str,
+        alias: str,
+        resource_hint: "ResourceHint | None" = None,
+        resource_group: str | None = None,
+    ) -> ModelVersion:
         """Creates a ModelReference using an alias identifier.
 
         Parameters
@@ -181,6 +232,11 @@ class ModelReference:
             The name of the model.
         alias
             The alias string. Must be non-empty.
+        resource_hint
+            Whether this model loading is bound by CPU, I/O, or GPU.
+        resource_group
+            The resource group for the model: this is used to isolate execution
+            onto a separate pod (or set of nodes), such as on a GPU-enabled node.
 
         Returns
         -------
@@ -195,11 +251,12 @@ class ModelReference:
         Examples
         --------
         >>> model = ModelReference.from_alias("fraud_model", "latest")
+        >>> model = ModelReference.from_alias("fraud_model", "latest", resource_hint="gpu", resource_group="gpu-group")
         """
         if not alias:
             raise ValueError("Alias must be a non-empty string.")
 
-        mr = ModelReference(name=name, alias=alias)
+        mr = ModelReference(name=name, alias=alias, resource_hint=resource_hint, resource_group=resource_group)
         return mr.model_version
 
 
