@@ -52,6 +52,14 @@ class ModelEncoding(str, Enum):
     SAFETENSOR = "MODEL_ENCODING_SAFETENSORS"
 
 
+class ModelClass(str, Enum):
+    CLASSIFICATION = "classification"
+    REGRESSION = "regression"
+    CLUSTERING = "clustering"
+    DIMENSIONALITY_REDUCTION = "dimensionality_reduction"
+    EMBEDDING = "embedding"
+
+
 @dataclass
 class ModelRunCriterion:
     direction: Literal["max", "min"]
@@ -391,13 +399,33 @@ class ModelAttributeExtractor:
         return input_schema, output_schema
 
     @staticmethod
-    def infer_model_type(model: Any) -> Optional[ModelType]:
+    def infer_model_type(model: Any) -> Tuple[Optional[ModelType], Optional[ModelClass]]:
+        # ONNX - check early since ONNX models are commonly wrapped
+        try:
+            import onnx  # pyright: ignore[reportMissingImports]
+
+            if isinstance(model, onnx.ModelProto):
+                return ModelType.ONNX, None
+            # Check if model has a wrapped ONNX ModelProto (e.g., model._model)
+            if hasattr(model, "_model") and isinstance(model._model, onnx.ModelProto):
+                return ModelType.ONNX, None
+        except ImportError:
+            pass
+
+        try:
+            import onnxruntime  # pyright: ignore[reportMissingImports]
+
+            if isinstance(model, onnxruntime.InferenceSession):
+                return ModelType.ONNX, None
+        except ImportError:
+            pass
+
         # PYTORCH
         try:
             import torch.nn as nn  # pyright: ignore[reportMissingImports]
 
             if isinstance(model, nn.Module):
-                return ModelType.PYTORCH
+                return ModelType.PYTORCH, None
         except ImportError:
             pass
 
@@ -405,11 +433,16 @@ class ModelAttributeExtractor:
         try:
             import xgboost as xgb  # pyright: ignore[reportMissingImports]
 
+            if isinstance(model, xgb.XGBClassifier):
+                return ModelType.XGBOOST, ModelClass.CLASSIFICATION
+            if isinstance(model, xgb.XGBRegressor):
+                return ModelType.XGBOOST, ModelClass.REGRESSION
+
             if isinstance(model, (xgb.XGBModel, xgb.Booster)):
-                return ModelType.XGBOOST
+                return ModelType.XGBOOST, None
             # Also check for XGBoost sklearn API
             if hasattr(model, "__class__") and "xgboost" in model.__class__.__module__:
-                return ModelType.XGBOOST
+                return ModelType.XGBOOST, None
         except ImportError:
             pass
 
@@ -418,9 +451,9 @@ class ModelAttributeExtractor:
             import lightgbm as lgb  # pyright: ignore[reportMissingImports]
 
             if isinstance(model, (lgb.LGBMModel, lgb.Booster)):
-                return ModelType.LIGHTGBM
+                return ModelType.LIGHTGBM, None
             if hasattr(model, "__class__") and "lightgbm" in model.__class__.__module__:
-                return ModelType.LIGHTGBM
+                return ModelType.LIGHTGBM, None
         except ImportError:
             pass
 
@@ -428,32 +461,20 @@ class ModelAttributeExtractor:
         try:
             import catboost as cb  # pyright: ignore[reportMissingImports]
 
-            # CatBoost has various model classes
-            if hasattr(model, "__class__") and "catboost" in model.__class__.__module__:
-                return ModelType.CATBOOST
-            # Common CatBoost classes
+            # Common CatBoost classes - check specific types first
             try:
-                if isinstance(model, (cb.CatBoost, cb.CatBoostClassifier, cb.CatBoostRegressor)):
-                    return ModelType.CATBOOST
+                if isinstance(model, cb.CatBoostClassifier):
+                    return ModelType.CATBOOST, ModelClass.CLASSIFICATION
+                if isinstance(model, cb.CatBoostRegressor):
+                    return ModelType.CATBOOST, ModelClass.REGRESSION
+
+                if isinstance(model, (cb.CatBoost)):
+                    return ModelType.CATBOOST, None
             except (AttributeError, NameError):
                 pass
-        except ImportError:
-            pass
-
-        # ONNX - do we need this one?
-        try:
-            import onnx  # pyright: ignore[reportMissingImports]
-
-            if isinstance(model, onnx.ModelProto):
-                return ModelType.ONNX
-        except ImportError:
-            pass
-
-        try:
-            import onnxruntime  # pyright: ignore[reportMissingImports]
-
-            if isinstance(model, onnxruntime.InferenceSession):
-                return ModelType.ONNX
+            # CatBoost has various model classes - generic fallback
+            if hasattr(model, "__class__") and "catboost" in model.__class__.__module__:
+                return ModelType.CATBOOST, None
         except ImportError:
             pass
 
@@ -462,10 +483,10 @@ class ModelAttributeExtractor:
             import sklearn.base  # pyright: ignore[reportMissingImports]
 
             if isinstance(model, sklearn.base.BaseEstimator):
-                return ModelType.SKLEARN
+                return ModelType.SKLEARN, None
 
             if hasattr(model, "__class__") and "sklearn" in model.__class__.__module__:
-                return ModelType.SKLEARN
+                return ModelType.SKLEARN, None
         except ImportError:
             pass
 
@@ -474,10 +495,10 @@ class ModelAttributeExtractor:
             import tensorflow as tf  # pyright: ignore[reportMissingImports]
 
             if isinstance(model, tf.keras.Model):
-                return ModelType.TENSORFLOW
+                return ModelType.TENSORFLOW, None
             if hasattr(model, "__class__") and "tensorflow" in model.__class__.__module__:
-                return ModelType.TENSORFLOW
+                return ModelType.TENSORFLOW, None
         except ImportError:
             pass
 
-        return None
+        return None, None
