@@ -173,6 +173,14 @@ def get_registered_types(scope_to: Path, failed: List[FailedImport]) -> UpsertGr
     named_queries: list[UpsertNamedQueryGQL] = []
     for named_query in NAMED_QUERY_REGISTRY.values():
         if named_query.filename is None or _is_relative_to(Path(named_query.filename), scope_to):
+            # only try to convert if there are no errors
+            if not named_query.errors:
+                try:
+                    named_queries.append(convert_type_to_gql(named_query, path_prefix=path_prefix_to_remove))
+                except Exception as e:
+                    failed.append(build_failed_import(e, f"named query '{named_query.name}'"))
+
+            # named_query.errors can go from empty to non-empty after conversion
             if named_query.errors:
                 code_object_diagnostics[named_query.filename or ""].extend(
                     [
@@ -185,15 +193,17 @@ def get_registered_types(scope_to: Path, failed: List[FailedImport]) -> UpsertGr
                         for error in named_query.errors
                     ]
                 )
-                continue
-            try:
-                named_queries.append(convert_type_to_gql(named_query, path_prefix=path_prefix_to_remove))
-            except Exception as e:
-                failed.append(build_failed_import(e, f"named query '{named_query.name}'"))
 
     model_references: list[UpsertModelReferenceGQL] = []
     for mr in MODEL_REFERENCE_REGISTRY.values():
         if mr.filename is None or _is_relative_to(Path(mr.filename), scope_to):
+            # similar logic to named_queries above
+            if not mr.errors:
+                try:
+                    model_references.append(convert_type_to_gql(mr, path_prefix=path_prefix_to_remove))
+                except Exception as e:
+                    failed.append(build_failed_import(e, f"model reference '{mr.name}'"))
+
             if mr.errors:
                 code_object_diagnostics[mr.filename or ""].extend(
                     [
@@ -206,11 +216,6 @@ def get_registered_types(scope_to: Path, failed: List[FailedImport]) -> UpsertGr
                         for error in mr.errors
                     ]
                 )
-                continue
-            try:
-                model_references.append(convert_type_to_gql(mr, path_prefix=path_prefix_to_remove))
-            except Exception as e:
-                failed.append(build_failed_import(e, f"model reference '{mr.name}'"))
 
     # online store configs
     for osc in ONLINE_STORE_CONFIG_REGISTRY.values():
