@@ -10,6 +10,7 @@ from chalk.sql._internal.integrations.clickhouse import ClickhouseSourceImpl
 from chalk.sql._internal.integrations.cloudsql import CloudSQLSourceImpl
 from chalk.sql._internal.integrations.databricks import DatabricksSourceImpl
 from chalk.sql._internal.integrations.dynamodb import DynamoDBSourceImpl
+from chalk.sql._internal.integrations.mssql import MSSQLSourceImpl
 from chalk.sql._internal.integrations.mysql import MySQLSourceImpl
 from chalk.sql._internal.integrations.postgres import PostgreSQLSourceImpl
 from chalk.sql._internal.integrations.redshift import RedshiftSourceImpl
@@ -464,6 +465,196 @@ def MySQLSource(
     )
 
 
+@overload
+def MSSQLSource() -> SQLSourceWithTableIngestProtocol:
+    """If you have only one MSSQL connection that you'd like
+    to add to Chalk, you do not need to specify any arguments
+    to construct the source in your code.
+
+    Returns
+    -------
+    SQLSourceWithTableIngestProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> mssql = MSSQLSource()
+    """
+    ...
+
+
+@overload
+def MSSQLSource(
+    *,
+    name: str,
+    engine_args: Optional[Dict[str, Any]] = ...,
+    async_engine_args: Optional[Dict[str, Any]] = ...,
+) -> SQLSourceWithTableIngestProtocol:
+    """If you have only one MSSQL integration, there's no need to provide
+    a distinguishing name.
+
+    But what happens when you have two data sources of the same kind?
+    When you create a new data source from your dashboard,
+    you have an option to provide a name for the integration.
+    You can then reference this name in the code directly.
+
+    Parameters
+    ----------
+    name
+        Name of the integration, as configured in your dashboard.
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine. These arguments will be
+        merged with any default arguments from the named integration.
+    async_engine_args
+        Additional arguments to use when constructing an async SQLAlchemy engine.
+
+    Returns
+    -------
+    SQLSourceWithTableIngestProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    >>> source = MSSQLSource(name="RISK")
+    """
+    ...
+
+
+@overload
+def MSSQLSource(
+    *,
+    name: str | None = ...,
+    host: str,
+    port: Union[int, str] = ...,
+    db: str = ...,
+    user: str = ...,
+    password: str = ...,
+    client_id: str = ...,
+    client_secret: str = ...,
+    tenant_id: str = ...,
+    engine_args: Optional[Dict[str, Any]] = ...,
+    async_engine_args: Optional[Dict[str, Any]] = ...,
+) -> SQLSourceWithTableIngestProtocol:
+    """
+    You can also configure the integration directly using environment
+    variables on your local machine or from those added through the
+    generic environment variable support (https://docs.chalk.ai/docs/env-vars).
+
+    Authentication Methods:
+    - SQL Authentication: Provide `user` and `password`
+    - Azure AD Managed Identity: Leave `user`, `password`, `client_id`, `client_secret`, and `tenant_id` empty
+    - Azure AD Service Principal: Provide `client_id`, `client_secret`, and `tenant_id`
+
+    Parameters
+    ----------
+    name
+        Name of the integration. Not required unless if this SQL Source is used within SQL File Resolvers.
+    host
+        Name of host to connect to.
+    port
+        The port number to connect to at the server host.
+    db
+        The database name.
+    user
+        MSSQL username to connect as (for SQL authentication).
+    password
+        The password to be used for SQL authentication.
+    client_id
+        Azure AD Client ID (for Service Principal authentication).
+    client_secret
+        Azure AD Client Secret (for Service Principal authentication).
+    tenant_id
+        Azure AD Tenant ID (for Service Principal authentication).
+    engine_args
+        Additional arguments to use when constructing the SQLAlchemy engine.
+    async_engine_args
+        Additional arguments to use when constructing an async SQLAlchemy engine.
+
+    Returns
+    -------
+    SQLSourceWithTableIngestProtocol
+        The SQL source for use in Chalk resolvers.
+
+    Examples
+    --------
+    SQL Authentication:
+    >>> import os
+    >>> mssql = MSSQLSource(
+    ...     host=os.getenv("MSSQL_HOST"),
+    ...     port=os.getenv("MSSQL_TCP_PORT"),
+    ...     db=os.getenv("MSSQL_DATABASE"),
+    ...     user=os.getenv("MSSQL_USER"),
+    ...     password=os.getenv("MSSQL_PWD"),
+    ... )
+
+    Managed Identity (running in Azure):
+    >>> mssql = MSSQLSource(
+    ...     host=os.getenv("MSSQL_HOST"),
+    ...     port=os.getenv("MSSQL_TCP_PORT"),
+    ...     db=os.getenv("MSSQL_DATABASE"),
+    ... )
+
+    Service Principal:
+    >>> mssql = MSSQLSource(
+    ...     host=os.getenv("MSSQL_HOST"),
+    ...     port=os.getenv("MSSQL_TCP_PORT"),
+    ...     db=os.getenv("MSSQL_DATABASE"),
+    ...     client_id=os.getenv("MSSQL_CLIENT_ID"),
+    ...     client_secret=os.getenv("MSSQL_CLIENT_SECRET"),
+    ...     tenant_id=os.getenv("MSSQL_TENANT_ID"),
+    ... )
+
+    >>> from chalk.features import online
+    >>> @online
+    ... def resolver_fn() -> User.name:
+    ...     return mssql.query_string("select name from users where id = 4").one()
+    """
+    ...
+
+
+def MSSQLSource(
+    *,
+    host: Optional[str] = None,
+    port: Optional[Union[int, str]] = None,
+    db: Optional[str] = None,
+    user: Optional[str] = None,
+    password: Optional[str] = None,
+    client_id: Optional[str] = None,
+    client_secret: Optional[str] = None,
+    tenant_id: Optional[str] = None,
+    name: Optional[str] = None,
+    engine_args: Optional[Dict[str, Any]] = None,
+    async_engine_args: Optional[Dict[str, Any]] = None,
+) -> SQLSourceWithTableIngestProtocol:
+    """Create a MSSQL data source. SQL-based data sources
+    created without arguments assume a configuration in your
+    Chalk Dashboard. Those created with the `name=` keyword
+    argument will use the configuration for the integration
+    with the given name. And finally, those created with
+    explicit arguments will use those arguments to configure
+    the data source. See the overloaded signatures for more
+    details.
+
+    Supports three authentication methods:
+    - SQL Authentication: user + password
+    - Azure AD Managed Identity: no credentials (automatic in Azure)
+    - Azure AD Service Principal: client_id + client_secret + tenant_id
+    """
+    return MSSQLSourceImpl(
+        host=host,
+        port=port,
+        db=db,
+        user=user,
+        password=password,
+        client_id=client_id,
+        client_secret=client_secret,
+        tenant_id=tenant_id,
+        name=name,
+        engine_args=engine_args,
+        async_engine_args=async_engine_args,
+    )
+
+
 def SQLiteInMemorySource(
     name: Optional[str] = None,
     engine_args: Optional[Dict[str, Any]] = None,
@@ -1757,6 +1948,8 @@ __all__ = (
     "DynamoDBSource",
     "FinalizedChalkQuery",
     "IncrementalSettings",
+    "MSSQLSource",
+    "MSSQLSourceImpl",
     "MySQLSource",
     "PostgreSQLSource",
     "RedshiftSource",
