@@ -303,7 +303,7 @@ def get_sql_file_resolvers(
     """Iterate through all `.chalk.sql` filepaths, gather the sql strings, and get a resolver hopefully for each."""
     for dp, dn, fn in os.walk(os.path.expanduser(sql_file_resolve_location)):
         del dn  # unused
-        for f in fn:
+        for f in sorted(fn):  # Sort filenames for deterministic ordering
             filepath = os.path.join(dp, f)
             if not filepath.endswith(CHALK_SQL_FILE_RESOLVER_FILENAME_SUFFIX):
                 continue
