@@ -6,9 +6,18 @@ import functools
 import queue
 import threading
 from concurrent.futures import Future, ThreadPoolExecutor
-from typing import Any, AsyncIterable, AsyncIterator, Callable, Coroutine, Generic, Iterable, TypeVar, overload
-
-from typing_extensions import ParamSpec
+from typing import (
+    Any,
+    AsyncIterable,
+    AsyncIterator,
+    Callable,
+    Coroutine,
+    Generic,
+    Iterable,
+    ParamSpec,
+    TypeVar,
+    overload,
+)
 
 from chalk.utils.tracing import safe_activate_trace_context, safe_current_trace_context
 
