@@ -595,9 +595,9 @@ def _read_parquet(
         if source.startswith("file://"):
             # Polars cannot handle local file uris properly
             source = source[len("file://") :]
-        if source.startswith("gs://"):
+        if source.startswith("gcs://"):
             # Fsspec expects gcs instead of gs
-            source = "gcs://" + source[len("gs://") :]
+            source = "gs://" + source[len("gcs://") :]
 
     try:
         # Polars added a crash bug in polars 0.20 that isn't fixed until polars 1.8.0; this fallback case handles
