@@ -36,6 +36,8 @@ if TYPE_CHECKING:
     import azure.storage.blob
     import google.cloud.storage
     import pyarrow as pa
+    from azure.core.credentials import TokenCredential
+    from azure.core.credentials_async import AsyncTokenCredential
     from fsspec import AbstractFileSystem
     from mypy_boto3_s3.client import S3Client
 
@@ -632,6 +634,7 @@ class AzureBlobStorageClient(StorageClient):
         account_name: str,
         container_name: str,
         executor: ThreadPoolExecutor,
+        credential: TokenCredential | AsyncTokenCredential | None = None,
     ):
         super().__init__()
         try:
@@ -644,9 +647,11 @@ class AzureBlobStorageClient(StorageClient):
         self._container_client = blob_service_client.get_container_client(container_name)
         self._account_name = account_name
         self._executor = executor
+        if credential is None:
+            credential = blob_service_client.credential
         self.fs = adlfs.AzureBlobFileSystem(
             account_name=account_name,
-            credential=blob_service_client.credential,
+            credential=credential,  # pyright: ignore[reportArgumentType]
         )
 
     @override
