@@ -32,15 +32,6 @@ if can_use_ddtrace and can_use_datadog_statsd:
         if not ddtrace.tracer.enabled:
             yield
             return
-        if (current_ctx := ddtrace.tracer.current_trace_context()) is None:
-            yield
-            return
-        if (priority := current_ctx.sampling_priority) is not None and priority <= 0:
-            # If a priority is negative, then it won't be sampled
-            # See https://github.com/DataDog/dd-trace-py/blob/09edef713bf9f0ab30f554bf7765d7a7c2ed6f30/ddtrace/constants.py#L74
-            # Not sure what a priority=None means
-            yield
-            return
         if attributes is None:
             attributes = {}
         attributes = dict(attributes)
@@ -83,6 +74,7 @@ if can_use_ddtrace and can_use_datadog_statsd:
         headers = dict(input_headers)
         span = ddtrace.tracer.current_span()
         if span:
+            span.context.sampling_priority = 2
             span.set_tags({ddtrace.constants.SAMPLING_PRIORITY_KEY: 2})  # Ensure that sampling is enabled
             HTTPPropagator.inject(span.context, headers)
         return headers
